/*
 * main.c	für MSP430G2553

 Stoppuhr mit Berechnung und LCD-Ausgabe (2 Zeilen je 16 Zeichen)
 - der Geschwindigkeit[km/h] nach Vorgabe des Weges[m]	(Modus: Geschwindigkeit)
 - der Entfernung[m] nach der Schall-Laufzeit[ms]		(Modus: Gewitter)

 Wird vor dem Einschalten der Taster gedrückt gehalten, so befindet sich das Gerät im "Gewitter-Modus":
 beim Blitz wird der Taster gedrückt und bis zum Donner gehalten.
 Auf dem LCD wird die Entfernung in [m] angezeigt.

 Sonst ist das Gerät im "Geschwindigkeits-Modus":
 Aus der Zeit t während der der Taster gedrückt ist, wird v = s/t in [km/h] errechnet und angezeigt
 Fest vorgegeben ist z.Zt. s = 200[m].


 Hardware: MSP430G2553 und ein Taster an P1.1 (Port und Pin einstellbar in main.h)
 läuft mit 32768Hz Quarz, Timer A0 liefert Interrupts alle 0,9765625ms

 unter Zuhilfenahme des Beispielprogramms "19_UhrmitSetup" von Gerrit Buhe
 sowie weiterer Programmsegmente aus Gerrits uC-Kurs beim DARC Ortsverband H13, mni tnx!

 16.08.2013	v0.707b	DL9MWE
 */
#include	"main.h"

// globale Variablen
enum Status status = StandBy; 				// { StandBy, Messung, Stopped }
enum KeyStatus keyStatus = Reading; 		// { Reading, Debouncing }
enum Modus messModus = Geschwindigkeit; 	// { Geschwindigkeit, Gewitter }
volatile unsigned long int tickzaehler = 0; // 0,9765625ms
unsigned int uiv, uis; 						// Geschwindigkeit[km/h] und Distanz[m] als Integerwerte
float dt; 									// Zeit in Millisekunden bzw. Stunden

void main(void) {
	enum DispMessung dispMessung = Ja; // zu Beginn der Messung Meldung anzeigen

	vInit_cpu();
	vTimerA_init(); //Quelle 32.768kHz-Quarz-Oszillator
	vLCD_init(16); //initialisiere LCD-Modul 2x16Chars

	if (!(KeyPort & Key1Bit)) {
		messModus = Gewitter; // Taste beim Start gedrückt => Gewitter-Modus
	}

	vLCD_puts("Stoppuhr20130816");
	vLCD_gotoxy(0, 1);
	_bis_SR_register(SCG0 + SCG1 + GIE); //Schalte DCO und SMCLK ganz aus, erlaube Interrupts

	switch (messModus) {
	case Gewitter: // Im Gewitter-Modus auf das Loslassen der Taste warten
		vLCD_puts("Gewitter-Rechner");
		status = Messung;
		keyStatus = Debouncing;
		_bis_SR_register(CPUOFF); // Strom sparen
		while (status == Messung) // via Timerinterrupt Entprellen und warten bis Taste losgelassen
			_bis_SR_register(CPUOFF);
		break;
	case Geschwindigkeit:
		vLCD_puts("  v - Rechner   ");
		uis = 200; // Voreinstellung der Distanz s
		break;
	default:
		break;
	}

	status = StandBy;
	_bis_SR_register(CPUOFF);

	while (42) { // Endlosschleife
		switch (status) { // { StandBy, Messung, Stopped }
		case Stopped:
			_bic_SR_register(GIE); // nicht weiterzaehlen: disable interrupts
			dt = (float) tickzaehler * 0.9765625; // Zeit in Millisekunden
			vCalculate(); // Wert (v oder s) errechnen
			vDisplay_update(); //  und anzeigen
			status = StandBy;
			dispMessung = Ja;
			_bis_SR_register(GIE); // auf nächste Messung warten: enable interrupts
			break;
		case Messung:
			if (dispMessung == Ja) { // "Messung"-Meldung einmal zu Beginn der Messung anzeigen
				dispMessung = Nein;
				vLCD_gotoxy(0, 0);
				vLCD_puts("    Messung     ");
				vLCD_gotoxy(0, 1);
				switch (messModus) {
				case Gewitter:
					vLCD_puts(" Gewitter-Modus ");
					break;
				case Geschwindigkeit:
					vLCD_puts("Geschwindigkeit ");
					break;
				default:
					break;
				}
			}
		default:
			break;
		}

		// jetzt wieder schlafen bis zum nächsten Interrupt
		_bis_SR_register(CPUOFF);
	}
}

void vInit_cpu() {
	WDTCTL = WDTPW + WDTHOLD; //stoppe Watchdog Timer

	//initialize Basic Clock Module, damit es 32.768kHz-Quarz verwendet
	DCOCTL = 0;
	BCSCTL1 = XT2OFF;
	BCSCTL2 = SELM_3 + SELS; //LFXT1CLK ist MCLK und SMCLK Quelle, (S)MCLK-divider = 1, internal DCO resistor
	BCSCTL3 = XCAP_3; //verwende 32.768kHz Xtal, Cs=12.5pF

	//initialisiere Port für Taster
	P1REN |= KeySetup; //schalte Widerstand zum Port-Pin
	P1OUT |= KeySetup; //andere Seite des R gegen Vcc --> Pull up
}

void vDisplay_update() {
	vLCD_clear();
	vLCD_puts("t");
	vLCD_putli(tickzaehler);
	vLCD_puts("[s]");
	vLCD_gotoxy(0, 1);
	switch (messModus) {
	case Gewitter:
		vLCD_puts("s=");
		vLCD_puti(uis);
		vLCD_puts("[m]");
		break;
	case Geschwindigkeit:
		vLCD_puts("v");
		vLCD_puti(uiv);
		vLCD_puts(" km/h");
		vLCD_puts(" s");
		vLCD_puti(uis);
		break;
	default:
		break;
	}

}

void vCalculate(void) {
	switch (messModus) {
	case Geschwindigkeit:
		// Im Geschwindigkeits-Modus v berechnen
		//	v = s / t
		uiv = (unsigned int) ((float) uis / dt * 3600.0);
		break;
	case Gewitter:
		// Im Gewitter-Modus Entfernung berechnen
		// Entfernung[m] = dt[Millisekunden] * Schallgeschwindigkeit[m/s] / 1000.
		//	343 [m/s] gilt für 20°C trockene Luft
		//	korrekter wäre eine Näherungsgleichung: c ~= (331,5  +  0,6 * Grad Celsius) [m]
		//	Diese Näherung gilt im Temperaturbereich −20 °C bis +40 °C mit einer Genauigkeit > 99,8 %.
		uis = (unsigned int) (dt * 0.343);
		break;
	default:
		break;
	}
}

// Ende main.c

